/*
  MongoDB database create script

  Note:
    It is not strictly necessary to pre create the database and
    collections in MongoDB. The database is automatically created when 
    first used (connected) and a collection is automatically created
    at the first insert request. However, this provides external 
    visibility for the "schema" which we use
*/

var dbName = 'ECDB_master';

var collList = [
    'COUNTER',
    'FILEDETAIL',
    'LOG',
    'EVENT',
    'ENDEAVOURJOB',
    'JOBSESSION',
    'JOBLOG',
    'ACCESSLOG',
    'NOTIFICATION',
    'REPORTINSTANCE',
    'REPORT',
    'RESOURCEPOOL',
    'ROLE',
    'USER',
    'ASSET',
    'BLOB',
    'IBMFCM',

    /* CHANGE LOG 2.4 */ 
    'APPLICATIONSERVER',
    'APPLICATIONCREDENTIAL',
    'LDAP',
    'TENANT',
    'RBACENTITY',
    'PRODUCTCONFIG',
];

/* Set logging level */
db.adminCommand({setParameter:1, logLevel:1})

/* Removed database drop step. Should be done independently */
/* Get to our database */
db = db.getMongo().getDB(dbName);

/* Then, create all our collections */
for (var i=0; i<collList.length; i++) {
  print("Creating collection: "+collList[i]);
  db.createCollection(collList[i]);
}

/* Finally, create required indexes for our collections */
print("Creating indexes for FILEDETAIL collection...");
db.FILEDETAIL.createIndex({key : 1});
db.FILEDETAIL.createIndex({rkey : 1});
db.FILEDETAIL.createIndex({uniqueFilename : 1});
db.FILEDETAIL.createIndex({entityId : 1});
db.FILEDETAIL.createIndex({nodeId : 1});
db.FILEDETAIL.createIndex({fileSize : -1});
db.FILEDETAIL.createIndex({fileSizeUsed : -1});
db.FILEDETAIL.createIndex({fileExtension : 1});
db.FILEDETAIL.createIndex({mtime : 1});
db.FILEDETAIL.createIndex({atime : 1});
db.FILEDETAIL.createIndex({ctime : 1});
db.FILEDETAIL.createIndex({type : 1});
db.FILEDETAIL.createIndex({volumeId : 1});
db.FILEDETAIL.createIndex({snapshotId : 1});
db.FILEDETAIL.createIndex({parentPath : 1});
db.FILEDETAIL.createIndex({virtualMachineID : 1});
db.FILEDETAIL.createIndex({virtualServerId : 1});
db.FILEDETAIL.createIndex({dataStoreId : 1});
db.FILEDETAIL.createIndex({node : 1});

print("Creating indexes for LOG collection...");
/*********************************************************/
/* comment out some index key to reduce the memory usage */
/*********************************************************/
db.runCommand({"convertToCapped":"LOG", size:5000000000});
db.LOG.createIndex({type : 1});
db.LOG.createIndex({logTime : 1});

print("Creating indexes for JOBLOG collection...");
/*********************************************************/
/* comment out some index key to reduce the memory usage */
/*********************************************************/
db.JOBLOG.createIndex({logTime : 1});
db.JOBLOG.createIndex({entityId : 1});
db.JOBLOG.createIndex({logTime : 1, entityId : 1});
db.JOBLOG.createIndex({type : 1});
db.JOBLOG.createIndex({jobsessionId : 1});
db.JOBLOG.createIndex({logClass : 1}); 
db.JOBLOG.createIndex({level : 1}); 
db.JOBLOG.createIndex({expiresAt : 1}, {expireAfterSeconds : 0});

print("Creating indexes for ACCESSLOG collection...");
db.runCommand({"convertToCapped":"ACCESSLOG", size:500000000});
db.ACCESSLOG.createIndex({logTime : 1});
db.ACCESSLOG.createIndex({entityId : 1});
db.ACCESSLOG.createIndex({logTime : 1, entityId : 1});
db.ACCESSLOG.createIndex({type : 1});

print("Creating indexes for LOGMESSAGE collection...");
db.LOGMESSAGE.createIndex({name : 1}, {unique : true});

print("Creating indexes for EVENT collection...");
/*********************************************************/
/* comment out some index key to reduce the memory usage */
/*********************************************************/
db.EVENT.createIndex({entityId : 1});
db.EVENT.createIndex({type : 1});
db.EVENT.createIndex({category : 1});
db.EVENT.createIndex({eventTime : -1});
db.EVENT.createIndex({expiresAt : 1}, {expireAfterSeconds : 0});

print("Creating indexes for NOTIFICATION collection...");
db.NOTIFICATION.createIndex({category : 1});
db.NOTIFICATION.createIndex({type : 1});
db.NOTIFICATION.createIndex({subType : 1});
db.NOTIFICATION.createIndex({notificationTime : -1});
db.NOTIFICATION.createIndex({name : 1});
db.NOTIFICATION.createIndex({dataSource : 1});
db.NOTIFICATION.createIndex({expiresAt : 1}, {expireAfterSeconds : 0});

print("Creating indexes for JOBSESSION collection...");
db.JOBSESSION.createIndex({type : 1, subType : 1});
db.JOBSESSION.createIndex({sessionId : 1, taskId : 1});
db.JOBSESSION.createIndex({start : 1, end : 1});
db.JOBSESSION.createIndex({jobName : 1});
db.JOBSESSION.createIndex({jobKey : 1});
db.JOBSESSION.createIndex({jobId : 1});
db.JOBSESSION.createIndex({status : 1});
db.JOBSESSION.createIndex({duration : 1});
db.JOBSESSION.createIndex({results : 1});
db.JOBSESSION.createIndex({index : 1});
db.JOBSESSION.createIndex({resourceId : 1});
db.JOBSESSION.createIndex({trackingId : 1});
db.JOBSESSION.createIndex({serviceId : 1});
db.JOBSESSION.createIndex({description : 1});
/* CHANGE LOG 2.4 */
db.JOBSESSION.createIndex({displayName : 1});
/* CHANGE LOG 2.5.1 */
db.JOBSESSION.createIndex({nextAction : 1});


print("Creating indexes for ENDEAVOURJOB collection...");
db.ENDEAVOURJOB.createIndex({name : 1});
db.ENDEAVOURJOB.createIndex({jobKey : 1});
db.ENDEAVOURJOB.createIndex({entityId : 1, nextRun : 1});
db.ENDEAVOURJOB.createIndex({type : 1});
db.ENDEAVOURJOB.createIndex({subType : 1});
db.ENDEAVOURJOB.createIndex({serviceId : 1});
db.ENDEAVOURJOB.createIndex({lastRun : 1});
db.ENDEAVOURJOB.createIndex({nextRun : 1});
db.ENDEAVOURJOB.createIndex({status : 1});
db.ENDEAVOURJOB.createIndex({lastSessionStatus : 1});
db.ENDEAVOURJOB.createIndex({lastSessionDuration : 1});

print("Creating indexes for SCHEDULE collection...");
db.SCHEDULE.createIndex({entityId : 1});
db.SCHEDULE.createIndex({name : 1});
db.SCHEDULE.createIndex({category : 1});
db.SCHEDULE.createIndex({type : 1});

print("Creating indexes for ENDEAVOURTIRGGERJOB collection...");
db.ENDEAVOURTRIGGERJOB.createIndex({name : 1});
db.ENDEAVOURTRIGGERJOB.createIndex({entityId : 1});
db.ENDEAVOURTRIGGERJOB.createIndex({category : 1});
db.ENDEAVOURTRIGGERJOB.createIndex({triggerId : 1});
db.ENDEAVOURTRIGGERJOB.createIndex({jobId : 1});

print("Creating indexes for USER collection...");
db.USER.createIndex({entityId : 1}, {unique : true});
/* CHANGE LOG 2.4 */
db.USER.createIndex({tenantId : 1, name : 1}, {unique : true});

print("Creating indexes for RESOURCEPOOL collection...");
db.RESOURCEPOOL.createIndex({entityId : 1}, {unique : true});
db.RESOURCEPOOL.createIndex({name : 1}, {unique : true});

print("Creating indexes for ROLE collection...");
db.ROLE.createIndex({entityId : 1}, {unique : true});
db.ROLE.createIndex({name : 1}, {unique : true});

print("Creating indexes for DPX collection...");
db.DPX.createIndex({entityId : 1}, {unique : true});
db.DPX.createIndex({name : 1}, {unique : true});

print("Creating indexes for HYPERV collection...");
db.HYPERVISOR.createIndex({entityId : 1}, {unique : true});
db.HYPERVISOR.createIndex({name : 1}, {unique : true});
db.HYPERVISOR.createIndex({type : 1});

print("Creating indexes for IBMSVC collection...");
db.IBMSVC.createIndex({name : 1}, {unique : true});

print("Creating indexes for EMCVNX collection...");
db.EMCVNX.createIndex({name : 1}, {unique : true});

print("Creating indexes for NETAPP collection...");
db.NETAPP.createIndex({name : 1}, {unique : true});

print("Creating indexes for VCLOUD collection...");
db.VCLOUD.createIndex({name : 1}, {unique : true});

print("Creating indexes for VSPHERE collection...");
db.VSPHERE.createIndex({name : 1}, {unique : true});

print("Creating indexes for SMTP collection...");
db.SMTP.dropIndex('name_1');
db.SMTP.createIndex({name : 1}, {unique : false});

print("Creating indexes for ENDEAVOURPOLICY collection...");
/* CHANGE LOG 2.4 */
db.ENDEAVOURPOLICY.update({"tenantId":{$exists: false}},{ $set: { "tenantId": NumberLong(1000) } }, { multi: true })
db.ENDEAVOURPOLICY.createIndex({tenantId : 1, name : 1, serviceId : 1}, {unique : true});

print("Creating indexes for ENDEAVOURJOB collection...");
/* CHANGE LOG 2.4 */
db.ENDEAVOURJOB.update({"tenantId":{$exists: false}},{ $set: { "tenantId": NumberLong(1000) } }, { multi: true })
db.ENDEAVOURJOB.createIndex({tenantId : 1, name : 1, serviceId : 1}, {unique : true});
db.ENDEAVOURJOB.createIndex({displayName : 1});

print("Creating indexes for ENDEAVOURTASK collection...");
db.ENDEAVOURTASK.createIndex({name : 1, type : 1}, {unique : false});

print("Creating indexes for REPORTINSTANCE collection...");
db.REPORTINSTANCE.createIndex({name : 1}, {unique : true});
db.REPORTINSTANCE.createIndex({fileDate : 1}, {unique : false});

print("Creating indexes for REPORT collection...");
db.REPORT.createIndex({entityId : 1}, {unique :  true});
db.REPORT.createIndex({name : 1}, {unique : true});
db.REPORT.createIndex({parentReportId : 1},{unique : false});

print("Creating indexes for ASSET collection...");
db.ASSET.createIndex({entityId : 1}, {unique :  true});
db.ASSET.createIndex({name : 1}, {unique : false});
db.ASSET.createIndex({category : 1}, {unique : false});
db.ASSET.createIndex({subCategory : 1}, {unique : false});
db.ASSET.createIndex({product : 1}, {unique : false});
db.ASSET.createIndex({version : 1}, {unique : false});
db.ASSET.createIndex({minProductVersion : 1}, {unique : false});
db.ASSET.createIndex({maxProductVersion : 1}, {unique : false});

print("Creating indexes for BLOB collection...");
db.BLOB.createIndex({entityId : 1}, {unique :  true});
db.BLOB.createIndex({name : 1}, {unique : false});


print("Creating indexes for IBMFCM collection...");
db.IBMFCM.createIndex({entityId : 1}, {unique :  true});
db.IBMFCM.createIndex({name : 1}, {unique : true});
db.IBMFCM.createIndex({uniqueId : 1}, {unique : true});
db.IBMFCM.createIndex({siteId : 1}, {unique : false});

print("Creating indexes for SITE collection...");
db.SITE.createIndex({name : 1}, {unique : true});

print("Creating indexes for STORAGEPROFILE collection...");
db.STORAGEPROFILE.createIndex({name : 1, type: 1 }, {unique : true});


print("Creating indexes for VMCREDENTIAL collection...");
db.VMCREDENTIAL.createIndex({entityId : 1}, {unique :  true});

print("Creating indexes for APPLICATIONSERVER collection...");
db.APPLICATIONSERVER.createIndex({entityId : 1}, {unique :  true});

print("Creating indexes for APPLICATIONCREDENTIAL collection...");
db.APPLICATIONCREDENTIAL.createIndex({entityId : 1}, {unique :  true});

print("Creating indexes for IBMXIV collection...");
db.IBMXIV.createIndex({name : 1}, {unique : true});
db.COUNTER.insert({_id:"IBMXIV", "lastSeqNum" : 1000 });

/* CHANGE LOG 2.2 */
/* from load.js */
db.COUNTER.insert({_id:"IBMFCM", "lastSeqNum" : 1000});

db.COUNTER.insert({_id:"IBMSVC", "lastSeqNum" : 1000 });

db.COUNTER.insert({_id:"ENDEAVOURTRIGGERJOB", "lastSeqNum" : 1000 });

db.COUNTER.insert({_id:"SECURITYEVENT", "lastSeqNum" : 1000 });
db.COUNTER.insert({_id:"SCHEDULE", "lastSeqNum" : 1000 });
db.COUNTER.insert({_id:"USER", "lastSeqNum" : 1000 });
db.COUNTER.insert({_id:"ROLE", "lastSeqNum" : 1000 });
db.COUNTER.insert({_id:"RESOURCEPOOL", "lastSeqNum" : 1000 });

/* CHANGE LOG 2.4 */
print("Creating indexes for TENANT collection...");
db.TENANT.createIndex({entityId : 1}, {unique :  true});
db.TENANT.createIndex({name : 1}, {unique :  true});

print("Creating indexes for RBACENTITY collection...");
db.RBACENTITY.createIndex({name : 1}, {unique :  true});

db.COUNTER.insert({_id:"APPLICATIONSERVER", "lastSeqNum" : 1000 });

db.COUNTER.insert({_id:"APPLICATIONCREDENTIAL", "lastSeqNum" : 1000 });

db.COUNTER.insert({_id:"TENANT", "lastSeqNum" : 1001 });

print("Updating indexes for LDAP collection...");
db.LDAP.dropIndex({name : 1});
db.LDAP.createIndex({tenantId : 1}, {unique : true});
db.LDAP.createIndex({entityId : 1}, {unique :  true});
db.USER.update({"tenantId":{$exists: false}},{ $set: { "tenantId": NumberLong(1000) } }, { multi: true })
db.LDAP.update({"tenantId":{$exists: false}},{ $set: { "tenantId": NumberLong(1000) } }, { multi: true })
db.COUNTER.insert({_id:"LDAP", "lastSeqNum" : 1010 });

print("Creating indexes for CONSTANT collection...");
db.CONSTANT.createIndex({name : 1, type : 1, constName : 1}, {unique : true});
db.CONSTANT.createIndex({entityId : 1}, {unique :  true});
db.COUNTER.insert({_id:"CONSTANT", "lastSeqNum" : 1000 });

/* CHANGE LOG 2.5 */
print("Updating indexes for EVENT collection...");
db.EVENT.createIndex({userId : 1, user : 1});

print("Updating indexes for ACCESSLOG collection...");
db.ACCESSLOG.createIndex({userId : 1, user : 1});

/* CHANGE LOG 2.5.1 */
print("Renaming ECXALL resource pool...");
db.RESOURCEPOOL.update({"name": "ECXALL"}, {$set: {"name": "All Resources"}})

/* CHANGE LOG 2.5.2 */
print("Updating indexes for ENDEAVOURPOLICY collection...");
db.ENDEAVOURPOLICY.createIndex({displayName : 1});
db.ENDEAVOURPOLICY.createIndex({comment : 1});
db.ENDEAVOURPOLICY.createIndex({creationTime : 1});
db.ENDEAVOURPOLICY.createIndex({lastUpdated : 1});
db.ENDEAVOURPOLICY.createIndex({policyKey : 1});

/* CHANGE LOG 2.6 */
print("Updating indexes for ENDEAVOURPOLICY collection...");
db.ENDEAVOURPOLICY.createIndex({applicationType : 1});
db.ENDEAVOURPOLICY.createIndex({subType : 1});

print("Updating indexes for JOBSESSION collection...");
db.JOBSESSION.createIndex({sessionKey : 1, taskKey : 1}, {unique : true, sparse: true});

print("Updating ENDEAVOURJOB collection...");
db.ENDEAVOURJOB.update({"type":"protection", "actions.actionType": "START"},{$set:{"actions.$.parameters.actionname.promptText":"SLA Policy"}}, {multi: true});

/* CHANGE LOG 3.0 */
 
dir='/opt/ECX/tools/dbScripts'
cd(dir)
pwd=pwd()
print('Current directory is :' + pwd)

print("Updating site to primary...");
db.SITE.update({entityId:1000}, {"$set": {"name": "Primary", "description": "Primary Site", "loadBalanceRatio": NumberInt(50)}});

/* Storage Profiles */

print("Updating STORAGEPROFILe counters  ...");
/* 2000 - 2100 - reserved for canned entries */
db.COUNTER.update({ "_id" : "STORAGEPROFILE"}, {"lastSeqNum" : 2100}, {upsert:true});
db.COUNTER.update({ "_id" : "SUBPOLICY"}, {"lastSeqNum" : 2100}, {upsert:true});
var storageprofilenames = ["./GoldSLA.json", "./SilverSLA.json", "./BronzeSLA.json", "./ContainerSLA.json", "./EC2SLA.json"];
var storageprofiles = [];

for (i=0; i < storageprofilenames.length; i++) {
    storageprofiles.push(cat(storageprofilenames[i]));

}
for (i=0; i < storageprofiles.length; i++) {
    var x = JSON.parse(storageprofiles[i]);
    if (x["name"] !== "Container" && x["name"] !== "EC2") {
        var d = new Date();
        x["spec"].subpolicy[0].trigger.activateDate = d.getTime()
    }
    print("Inserting canned storageprofile:  " + x["name"]);
    db.STORAGEPROFILE.update({name : x["name"]}, {$set:x}, {upsert:true});
}


/* ### Identity ### */
print("Updating IDENTITY counters  ...");
db.COUNTER.update({ "_id" : "IDENTITYUSER"}, {"lastSeqNum" : 2100}, {upsert:true});
var identityfilenames = [];
var identities = [];

for (i=0; i < identityfilenames.length; i++) {
    identities.push(cat(identityfilenames[i]));

}
for (i=0; i < identities.length; i++) {
    var x = JSON.parse(identities[i]);
    print("Inserting canned identity:  " + x["name"]);
    db.IDENTITYUSER.update({name : x["name"]}, {$set:x}, {upsert:true});
}

print("Updating STORAGECOMMON counters  ...");
db.COUNTER.update({ "_id" : "STORAGECOMMON"}, {"lastSeqNum" : 2100}, {upsert:true});

print("Inserting Secondary site  ...");
db.SITE.insert({ "entityId" : NumberInt(2000), "name" : 'Secondary', "description" : 'secondary site', "storageType": "NONE", "defaultSite" : false, "loadBalanceRatio": NumberInt(50)});
print("Updating SITE counters  ...");
db.COUNTER.update({ "_id" : "SITE"}, {"lastSeqNum" : 2100}, {upsert:true});

print("MongoDB database create completed");
